import io
import sys
import typing as t

import PIL.Image

from ._base import ImagePrinterBase
from ._utils import get_cursor, set_cursor

_SIXEL_TEST = b'\x1bPq"1;1;1;1#0;2;0;0;0-#0@\x1b\\'


class SixelPrinter(ImagePrinterBase):
    colors: int
    max_width: int

    def __init__(self, colors: int = 64, max_width: int = 800):
        self.colors = colors
        self.max_width = max_width

    @classmethod
    def supported(cls) -> bool:
        if not sys.stdin.isatty():
            return False
        saved_cur = get_cursor()
        if saved_cur is None:
            return False
        set_cursor(1, 1)
        sys.stdout.buffer.write(_SIXEL_TEST)
        sys.stdout.buffer.flush()
        cur = get_cursor()
        supported = cur is not None and cur[0] > 1
        set_cursor(*saved_cur)
        return supported

    def _encode(self, img: PIL.Image.Image, buf: t.BinaryIO) -> None:
        w, h = img.size
        if w > self.max_width:
            img = img.resize((self.max_width, h * self.max_width // w))
            w, h = img.size
        sixel_img = img.convert("RGBA").convert("P", colors=self.colors)

        # write header
        # write image size
        buf.write(b'\x1bPq"1;1;%d;%d' % (w, h))

        # write palette
        colors: t.Mapping[t.Tuple[int, int, int, int], int] = sixel_img.palette.colors
        for i, color in enumerate(colors.keys()):
            r, g, b, a = color
            if a == 0:  # skip transparent colors
                continue
            r = r * 100 // 255
            g = g * 100 // 255
            b = b * 100 // 255
            buf.write(b"#%d;2;%d;%d;%d" % (i, r, g, b))

        # write color data
        img_bytes: bytes = sixel_img.tobytes()  # type: ignore

        sixel_rows = (h + 5) // 6

        for sixel_row in range(sixel_rows):
            color_rows = [
                bytearray(w) if a != 0 else None for _, _, _, a in colors.keys()
            ]
            for col in range(w):
                for sub_row in range(6):
                    row = sixel_row * 6 + sub_row
                    if row >= h:
                        continue
                    i = row * w + col
                    color_index = img_bytes[i]
                    color_row = color_rows[color_index]
                    if color_row is None:
                        continue
                    color_row[col] |= 1 << sub_row
            for i, color_row in enumerate(color_rows):
                if color_row is None:
                    continue
                for col in range(w):
                    color_row[col] += 63
                buf.write(b"#%d" % i)
                buf.write(color_row)
                buf.write(b"$")
            buf.write(b"-")
        buf.write(b"\x1b\\")

    def encode(self, img: PIL.Image.Image) -> bytes:
        buf = io.BytesIO()
        self._encode(img, buf)
        return buf.getvalue()

    def print(self, img: PIL.Image.Image) -> None:
        self._encode(img, sys.stdout.buffer)



import os
import typing as t

from .printers import ITerm2Printer, SixelPrinter
from .printers._base import ImagePrinterBase


def get_printer(override: t.Optional[str] = None) -> t.Optional[ImagePrinterBase]:
    override = override or os.getenv("TERMIMG_PRINTER")
    if override == "iterm2":
        return ITerm2Printer()
    elif override == "sixel":
        return SixelPrinter()

    if ITerm2Printer.supported():
        return ITerm2Printer()
    elif SixelPrinter.supported():
        return SixelPrinter()
    return None


__all__ = ["get_printer"]



import sys
import typing as t

import PIL.Image

from . import get_printer

def main():
    f: t.Union[t.BinaryIO, str] = sys.stdin.buffer
    if len(sys.argv) > 1:
        f = sys.argv[1]
    img = PIL.Image.open(f)
    img.load()
    printer = get_printer()
    if printer is None:
        print("No compatible image printer available.", file=sys.stderr)
        exit(1)
    printer.print(img)

if __name__ == "__main__":
    main()



import sys

import PIL.Image


class ImagePrinterBase:
    @classmethod
    def supported(cls) -> bool:
        raise NotImplementedError

    def encode(self, img: PIL.Image.Image) -> bytes:
        raise NotImplementedError

    def print(self, img: PIL.Image.Image) -> None:
        sys.stdout.buffer.write(self.encode(img))
        sys.stdout.buffer.write(b"\n")
        sys.stdout.flush()



from distutils.core import setup

setup(
    name="pytermimg",
    version="0.1.0b1",
    python_requires='>3.6',
    packages=["termimg", "termimg.printers"],
    license="MIT",
    long_description=open("README.md").read(),
    author="Yifan Gu",
    author_email="dev@yifangu.com",
    install_requires=["pillow>=8.3.1"]
)



import signal
import sys
import os
import time
import threading

import typing as t

try:
    import msvcrt
except:
    msvcrt = None
    import tty, termios


def _sigterm_after(timeout: float, state: t.Sequence[t.Any]):
    time.sleep(timeout)
    if state[0]:
        os.kill(os.getpid(), signal.SIGINT)


def read_until(delimiter: bytes, timeout: float) -> bytes:
    fd = 0
    attr: t.List[t.Any] = []
    if msvcrt is None:
        fd = sys.stdin.fileno()
        attr = termios.tcgetattr(fd)
        tty.setraw(fd)
    state = [True]
    threading.Thread(target=_sigterm_after, args=[timeout, state], daemon=True).start()
    buffer: bytes = b""
    try:
        while True:
            if msvcrt is not None:
                c: bytes = msvcrt.getch()  # type: ignore
            else:
                c = sys.stdin.buffer.read(1)
            buffer += c
            if c == delimiter:
                state[0] = False
                return buffer
    except KeyboardInterrupt:
        pass
    finally:
        if msvcrt is None:
            termios.tcsetattr(fd, termios.TCSADRAIN, attr)
        return buffer


def get_cursor() -> t.Optional[t.Tuple[int, int]]:
    sys.stdout.buffer.write(b"\033[6n")
    sys.stdout.buffer.flush()
    buffer = read_until(b"R", 1.0)
    if buffer.startswith(b"\x1b[") and buffer.endswith(b"R"):
        row, col = [int(x) for x in buffer[2:-1].split(b";")]
        return (row, col)
    return None


def set_cursor(row: int, col: int) -> None:
    sys.stdout.buffer.write(f"\033[{row};{col}H".encode("ascii"))
    sys.stdout.buffer.flush()


def save_position() -> None:
    sys.stdout.buffer.write(b"\x1b7")
    sys.stdout.buffer.flush()


def restore_position() -> None:
    sys.stdout.buffer.write(b"\x1b8")
    sys.stdout.buffer.flush()
